import re
from django.conf.urls import url
from .action_maps import detail_route, list_route


def detail_route_url(viewset, lookup_field, ignored=[]):
    model = viewset.queryset.model
    name = model._meta.model_name.lower()
    name_plural = model._meta.verbose_name_plural.lower()
    name_plural_spaceless = re.sub(r"\s+", "", name_plural)
    return url(
        regex=r"^{}/(?!({})/?$)(?P<{}>[^/.]+)/?$".format(
            name_plural_spaceless,
            "|".join([name for name in ignored]),
            lookup_field,
        ),
        view=viewset.as_view(actions=detail_route),
        name="{}-detail".format(name)
    )


def list_route_url(viewset):
    model = viewset.queryset.model
    name = model._meta.model_name.lower()
    name_plural = model._meta.verbose_name_plural.lower()
    name_plural_spaceless = re.sub(r"\s+", "", name_plural)
    return url(
        regex=r"^{}/?$".format(name_plural_spaceless),
        view=viewset.as_view(actions=list_route),
        name="{}-list".format(name)
    )


def make_urlpatterns(viewset, lookup_field, ignored=[]):
    return [
        list_route_url(viewset),
        detail_route_url(viewset, lookup_field, ignored),
    ]



"""
http://www.django-rest-framework.org/api-guide/content-negotiation/#setting-the-content-negotiation

https://stackoverflow.com/questions/45498989/django-rest-framework-output-in-json-to-the-browser-by-default

https://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html
"""
from rest_framework.negotiation import BaseContentNegotiation


class IgnoreClientContentNegotiation(BaseContentNegotiation):
    """
    A custom content negotiation class which ignores the client request when selecting the appropriate parser or renderer.
    """

    def select_parser(self, request, parsers):
        """
        Select the first parser in the `.parser_classes` list.
        """
        return parsers[0]

    def select_renderer(self, request, renderers, format_suffix):
        """
        Select the first renderer in the `.renderer_classes` list.
        """
        return (renderers[0], renderers[0].media_type)



class QuerystringFilter(object):
    def get_querystring_filter(self):
        field_names = self.get_field_names()
        filter_kwargs = {}
        for key, value in self.request.query_params.iteritems():
            if key in field_names:
                filter_kwargs[key] = value
        return filter_kwargs

    def get_queryset(self):
        queryset = super().get_queryset()
        querystring_filter = self.get_querystring_filter()
        queryset = queryset.filter(**querystring_filter)
        return queryset



"""
[The OPTIONS] method allows a client to determine the options and/or requirements associated with a resource, or the capabilities of a server, without implying a resource action or initiating a resource retrieval.
"""
from rest_framework.metadata import BaseMetadata


class NoMetaData(BaseMetadata):
    """
    A null metadata scheme that configures OPTIONS responses to be empty.

    To set that metadata class globally we can use the DEFAULT_METADATA_CLASS setting in Rest Framework:

    REST_FRAMEWORK = {
        'DEFAULT_METADATA_CLASS': 'yourapp.metadata.NoMetaData'
    }

    https://dbader.org/blog/django-rest-framework-options-response
    """    
    def determine_metadata(self, request, view):
        return None
